from typing import Any, Coroutine, TypeAlias, TypeVar, TYPE_CHECKING, Union
from pydantic import BaseModel

if TYPE_CHECKING:
    from yc_lockbox._models import YandexLockboxResponse, YandexLockboxError

T = TypeVar("T", bound=BaseModel)

YandexCloudGenericResponse: TypeAlias = (
    Coroutine[Any, Any, Union["YandexLockboxResponse", "YandexLockboxError"]]
    | Union["YandexLockboxResponse", "YandexLockboxError"]
)



import re

YC_LOCKBOX_BASE_URL = "https://lockbox.api.cloud.yandex.net/lockbox/v1"
YC_LOCKBOX_PAYLOAD_BASE_URL = "https://payload.lockbox.api.cloud.yandex.net/lockbox/v1"
YC_IAM_BASE_URL = "https://iam.api.cloud.yandex.net"
JWT_ALGORITHM = "PS256"

# regex patterns

IAM_TOKEN_REGEX = re.compile(r"t1\.[A-Z0-9a-z_-]+[=]{0,2}\.[A-Z0-9a-z_-]{86}[=]{0,2}")
OAUTH_TOKEN_REGEX = re.compile(r"y[0-3]_[a-zA-Z0-9_-]+")
LEGACY_OAUTH_TOKEN_REGEX = re.compile(r"AQ[a-zA-Z0-9_-]+")



import logging
from typing import Any, Iterator
from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, SecretStr, SecretBytes

from yc_lockbox._abc import AbstractYandexLockboxClient


logger = logging.getLogger(__name__)


class BaseDomainModel(BaseModel):
    client: AbstractYandexLockboxClient | None = None
    model_config: ConfigDict = ConfigDict(extra="ignore", arbitrary_types_allowed=True)

    def inject_client(self, client: AbstractYandexLockboxClient) -> None:
        """
        Inject initialized client for make operations via domain model.

        :param inject_client: An initialized instance of :class:`AbstractYandexLockboxClient`.
        """
        self.client = client


class BaseUpsertModel(BaseModel):
    model_config: ConfigDict = ConfigDict(extra="forbid", populate_by_name=True)


class BasePaginatedResponse(BaseDomainModel):
    next_page_token: str | None = Field(None, alias="nextPageToken")


class YandexLockboxError(BaseDomainModel):
    code: int
    message: str | None = None
    details: Any = None


class YandexLockboxResponse(BaseDomainModel):
    id: str | None = None
    done: bool = False
    description: str | None = None
    created_by: str | None = Field(None, alias="createdBy")
    created_at: datetime | None = Field(None, alias="createdAt")
    modified_at: datetime | None = Field(None, alias="modifiedAt")
    metadata: dict[str, str] | None = None


class IamTokenResponse(BaseDomainModel):
    token: str = Field(..., alias="iamToken")
    expires_at: datetime = Field(..., alias="expiresAt")


class SecretPayloadEntry(BaseDomainModel):
    key: str
    text_value: SecretStr | None = Field(None, alias="textValue")
    binary_value: SecretBytes | None = Field(None, alias="binaryValue")

    def reveal_text_value(self) -> str:
        """Reveal a text value."""
        if self.text_value is None:
            return None

        return self.text_value.get_secret_value()

    def reveal_binary_value(self) -> bytes:
        """Reveal a binary value."""
        if self.binary_value is None:
            return None

        return self.binary_value.get_secret_value()


class INewSecretPayloadEntry(BaseUpsertModel):
    key: str
    text_value: str | None = Field(None, alias="textValue")
    binary_value: str | None = Field(None, alias="binaryValue")


class INewSecret(BaseUpsertModel):
    folder_id: str = Field(..., alias="folderId")
    name: str | None = None
    description: str | None = None
    labels: dict[str, str] | None = {}
    kms_key_id: str | None = Field(None, alias="kmsKeyId")
    version_description: str | None = Field(None, alias="versionDescription")
    version_payload_entries: list[INewSecretPayloadEntry] = Field(..., alias="versionPayloadEntries")
    deletion_protection: bool = Field(False, alias="deletionProtection")


class IUpdateSecret(BaseUpsertModel):
    update_mask: str = Field(..., alias="updateMask", description="Comma-separated model field names to be updated.")
    name: str | None = None
    description: str | None = None
    labels: dict[str, str] | None = None
    deletion_protection: bool | None = Field(None, alias="deletionProtection")


class INewSecretVersion(BaseUpsertModel):
    description: str | None = None
    base_version_id: str | None = Field(None, alias="baseVersionId")
    payload_entries: list[INewSecretPayloadEntry] = Field(..., alias="payloadEntries")

    model_config: ConfigDict = ConfigDict(extra="forbid", populate_by_name=True)


class SecretPayload(BaseDomainModel):
    version_id: str = Field(..., alias="versionId")
    entries: list[SecretPayloadEntry]

    def __getitem__(self, key: str) -> SecretPayloadEntry | None:
        """Get entry by key. Dictionary like."""
        value: SecretPayloadEntry | None = self.get(key, default=None)

        if value is None:
            entries = ", ".join(map(lambda entry: entry.key, self.entries))
            raise KeyError(f"Entry with name {key} not exists. Available entries: {entries}")

        return value

    def get(self, key: str, default: Any = None) -> SecretPayloadEntry | None:
        """
        Get entry object from payload.

        :param key: Entry key (name).
        :param default: Default return value if key not exists.
        :raises KeyError: When key not exists in payload.
        """
        return next(filter(lambda entry: entry.key == key, self.entries), default)


class SecretVersion(BaseDomainModel):
    id: str
    status: str  # todo: enum
    description: str | None = None
    secret_id: str = Field(..., alias="secretId")
    created_at: datetime = Field(..., alias="createdAt")
    destroy_at: datetime | None = Field(None, alias="destroyAt")
    payload_entry_keys: list[str] | None = Field(None, alias="payloadEntryKeys")

    # TODO: implement ``is_current`` property if possible
    # there may be compatibility issues with the subquery
    # in different modes (synchronous, asynchronous)

    def cancel_version_destruction(self, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.cancel_secret_version_destruction(self.secret_id, self.id, **kwargs)

    def payload(self, **kwargs) -> SecretPayload | YandexLockboxError:
        return self.client.get_secret_payload(self.secret_id, self.id, **kwargs)

    def schedule_version_destruction(
        self, pending_period: int = 604800, **kwargs
    ) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.schedule_secret_version_destruction(self.secret_id, self.id, pending_period, **kwargs)


class Secret(BaseDomainModel):
    id: str
    status: str  # todo: enum
    name: str | None = None
    folder_id: str = Field(..., alias="folderId")
    created_at: datetime = Field(..., alias="createdAt")
    description: str | None = None
    kms_key_id: str | None = Field(None, alias="kmsKeyId")
    current_version: SecretVersion = Field(..., alias="currentVersion")
    deletion_protection: bool = Field(..., alias="deletionProtection")
    labels: dict[str, str] | None = None

    def activate(self, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.activate_secret(self.id, **kwargs)

    def add_version(self, version: INewSecretVersion, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.add_secret_version(self.id, version, **kwargs)

    def cancel_version_destruction(self, version_id: str, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.cancel_secret_version_destruction(self.id, version_id, **kwargs)

    def deactivate(self, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.deactivate_secret(self.id, **kwargs)

    def delete(self, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.delete_secret(self.id, **kwargs)

    def refresh(self, **kwargs) -> "Secret":
        return self.client.get_secret(self.id, **kwargs)

    def payload(self, version_id: str | None = None, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.get_secret_payload(self.id, version_id, **kwargs)

    def list_versions(
        self, page_size: int = 100, page_token: str | None = None, iterator: bool = False, **kwargs
    ) -> "SecretVersionsList" | Iterator[SecretVersion] | YandexLockboxError:
        return self.client.list_secret_versions(
            self.id, page_size=page_size, page_token=page_token, iterator=iterator, **kwargs
        )

    def schedule_version_destruction(
        self, version_id: str, pending_period: int = 604800, **kwargs
    ) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.schedule_secret_version_destruction(self.id, version_id, pending_period, **kwargs)

    def update(self, data: IUpdateSecret, **kwargs) -> YandexLockboxResponse | YandexLockboxError:
        return self.client.update_secret(self.id, data, **kwargs)


class SecretsList(BasePaginatedResponse):
    secrets: list[Secret] = []


class SecretVersionsList(BasePaginatedResponse):
    versions: list[SecretVersion] = []



#!/usr/bin/env python3

import sys

from os import path
from setuptools import find_packages, setup


try:
    from semantic_release import setup_hook

    setup_hook(sys.argv)
except ImportError:
    pass


cwd = path.abspath(path.dirname(__file__))


def build_description():
    with open("README.md", "r") as fh:
        long_description = fh.read()

    return long_description


def metadata():
    meta = {}
    with open(path.join(cwd, "yc_lockbox", "__init__.py"), "r") as fh:
        exec(fh.read(), meta)  # nosec
    return meta


def get_requirements(req_file: str | list[str] = "requirements.txt"):
    requirements_list = []

    def collect_requirements(f: str):
        nonlocal requirements_list

        with open(f) as requirements:
            for install in requirements:
                requirements_list.append(install.strip())

    if isinstance(req_file, list):
        for req in req_file:
            collect_requirements(req)
    else:
        collect_requirements(req_file)

    return requirements_list


def aio_requirements():
    return get_requirements("requirements.aio.txt")


meta = metadata()
readme = build_description()


def main() -> None:
    setup(
        name="yc-lockbox",
        keywords=["yandex", "cloud", "vault", "secrets", "lockbox"],
        platforms=["osx", "linux"],
        entry_points={},
        version=meta.get("__version__"),
        author=meta.get("__author__"),
        author_email=meta.get("__author_email__"),
        license=meta.get("__license__"),
        python_requires=">=3.10",
        packages=find_packages(),
        install_requires=get_requirements(),
        extras_require={
            "aio": aio_requirements(),
        },
        classifiers=[
            "License :: OSI Approved :: MIT License",
            "Intended Audience :: Information Technology",
            "Intended Audience :: System Administrators",
            "Intended Audience :: Developers",
            "Operating System :: OS Independent",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Topic :: Software Development :: Libraries",
            "Topic :: Software Development",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Programming Language :: Python :: 3.12",
            "Typing :: Typed",
        ],
    )


if __name__ == "__main__":
    main()



import logging
import requests
from typing import Any

from yc_lockbox._abc import AbstractHTTPAdapter
from yc_lockbox._models import YandexLockboxError
from yc_lockbox._types import T

logger = logging.getLogger(__name__)


class HTTPAdapter(AbstractHTTPAdapter):
    """
    An basic adapter for HTTP requests to origin.
    """

    @staticmethod
    def parse_response(
        response: requests.Response,
        response_model: T | None = None,
    ) -> T | Any:
        """
        Parse response from Origin.

        :param response: A HTTP-response object.
        :param response_model: Python object that will be used for transform response from origin.
        """
        match response.headers.get("Content-Type"):
            case "application/json":
                result = response.json()

                if response_model is not None:
                    if isinstance(result, list):
                        result = [response_model(**i) for i in result]
                        return result
                    return response_model(**result)

                return result
            case "text/plain":
                return response.text
            case _:
                return response.content

    @staticmethod
    def request(
        method: str,
        url: str,
        data: str | bytes | None = None,
        json: dict[str, Any] | None = None,
        headers: dict[str, Any] | None = None,
        params: dict[str, Any] | None = None,
        response_model: T | None = None,
        raise_for_status: bool = True,
        **kwargs,
    ) -> T | Any:
        """
        Method for make HTTP-requests.

        :param method: HTTP-method for request. Example: ``GET``, ``POST``.
        :param url: Request URL. Example ``https://api.example.com/users``.
        :param params: Dictionary request parameters to be sent in the query string.
        :param data: Dictionary, bytes, or file-like object to send in the body of the request.
        :param json: Any json compatible python object.
        :param headers: Dictionary of HTTP Headers to send with the request.
        :param response_model: Python object that will be used for transform response from origin.
        :param raise_client_errors: If set to ``True``, any client error (``4xx`` status code)
            will be throw exception.

        """

        response = requests.request(
            method=method, url=url, data=data, json=json, headers=headers, params=params, **kwargs
        )

        if response.status_code >= 400:
            logger.error(f"HTTP request failed with status {response.status_code}: {response.content.decode()}")

            if raise_for_status:
                response.raise_for_status()

            return __class__.parse_response(response, response_model=YandexLockboxError)

        return __class__.parse_response(response, response_model=response_model)


class AsyncHTTPAdapter(AbstractHTTPAdapter):
    """
    Another adapter for HTTP requests to origin.
    Similar to :class:`HTTPAdapter` but async.
    """

    @staticmethod
    async def parse_response(response: requests.Response, response_model: T | None = None) -> T | Any:
        """
        Parse response from Origin.

        :param response: A HTTP-response object.
        :param response_model: Python object that will be used for transform response from origin.
        """

        match response.headers.get("Content-Type"):
            case "application/json":
                result = await response.json()

                if response_model is not None:
                    if isinstance(result, list):
                        result = [response_model(**i) for i in result]
                        return result
                    return response_model(**result)

                return result
            case "text/plain":
                return await response.text()
            case _:
                return await response.read()

    @staticmethod
    async def request(
        method: str,
        url: str,
        data: str | bytes | None = None,
        json: dict[str, Any] | None = None,
        params: dict[str, Any] | None = None,
        headers: dict[str, Any] | None = None,
        response_model: T | None = None,
        raise_for_status: bool = True,
        **kwargs,
    ) -> T | Any:
        """
        Method for asynchronous make HTTP-requests.

        :param method: HTTP-method for request. Example: ``GET``, ``POST``.
        :param url: Request URL. Example ``https://api.example.com/users``.
        :param params: Dictionary request parameters to be sent in the query string.
        :param data: Dictionary, bytes, or file-like object to send in the body of the request.
        :param json: Any json compatible python object.
        :param headers: Dictionary of HTTP Headers to send with the request.
        :param response_model: Python object that will be used for transform response from origin.
        :param raise_client_errors: If set to ``True``, any client error (``4xx`` status code)
            will be throw exception.

        """
        try:
            import aiohttp
        except ImportError:
            raise ImportError(
                "Async mode is unavailable cause mandatory library ``aiohttp`` is not installed. "
                "Install ``aiohttp`` directly ``pip install aiohttp`` "
                "or use ``pip install yc-lockbox[aio]`` for resolve it."
            )

        async with aiohttp.ClientSession() as session:
            async with session.request(
                method=method, url=url, data=data, json=json, headers=headers, params=params, **kwargs
            ) as response:

                if response.status >= 400:
                    response_message = await response.read()
                    logger.error(f"HTTP request failed with status {response.status}: {response_message.decode()}")

                    if raise_for_status:
                        response.raise_for_status()

                    return await __class__.parse_response(response, response_model=YandexLockboxError)

                return await __class__.parse_response(response, response_model=response_model)
